package org.foxarmy.barcodescannerforemployees

import android.content.Context
import android.database.sqlite.SQLiteDatabase
import android.database.sqlite.SQLiteOpenHelper
import android.provider.BaseColumns
import android.util.Log
import java.io.File

object ProductContract {
    object ProductEntry : BaseColumns {
        const val TABLE_NAME = "products"
        const val PRODUCT_NAME = "name"
        const val PRODUCT_NET_WEIGHT = "net_weight"
        const val IMAGE_FILENAME = "image_filename"
    }
}

object ShelfContract {
    object ShelfEntry : BaseColumns {
        const val TABLE_NAME = "shelf"
        const val PRODUCT_ID = "product_id"
        const val EXPIRE_DATE = "expire_date"
    }
}

object CategoriesContract {
    object CategoryEntry : BaseColumns {
        const val TABLE_NAME = "categories"
        const val CATEGORY_NAME = "category_name"
    }
}

const val SQL_CREATE_PRODUCT_TABLE =
    "CREATE TABLE ${ProductContract.ProductEntry.TABLE_NAME} (" +
            "${BaseColumns._ID} INTEGER PRIMARY KEY," +
            "${ProductContract.ProductEntry.PRODUCT_NAME} TEXT," +
            "${ProductContract.ProductEntry.PRODUCT_NET_WEIGHT} REAL," +
            "${ProductContract.ProductEntry.IMAGE_FILENAME} TEXT)"

const val SQL_CREATE_SHELF_TABLE =
    "CREATE TABLE ${ShelfContract.ShelfEntry.TABLE_NAME} (" +
        "${ShelfContract.ShelfEntry.PRODUCT_ID} INTEGER," +
        "${ShelfContract.ShelfEntry.EXPIRE_DATE} DATE)"

const val SQL_CREATE_CATEGORIES_TABLE =
    "CREATE TABLE ${CategoriesContract.CategoryEntry.TABLE_NAME} (" +
            "${CategoriesContract.CategoryEntry.CATEGORY_NAME} TEXT)"

class DBStorageController(context: Context) : SQLiteOpenHelper(context, DATABASE_NAME, null, DATABASE_VERSION) {

    override fun onCreate(db: SQLiteDatabase) {
        db.execSQL(SQL_CREATE_PRODUCT_TABLE)
        db.execSQL(SQL_CREATE_SHELF_TABLE)
        db.execSQL(SQL_CREATE_CATEGORIES_TABLE)
    }

    override fun onUpgrade(p0: SQLiteDatabase?, p1: Int, p2: Int) {
        TODO("Not yet implemented")
    }

    public fun eraseAbstractProduct(db: SQLiteDatabase, id: Int, context: Context) {
        val projection = arrayOf(BaseColumns._ID,
            ProductContract.ProductEntry.IMAGE_FILENAME
        )

//        val cursor = db.query(ProductContract.ProductEntry.TABLE_NAME, projection, null, , null, null, null)
        val selection = "${BaseColumns._ID} = ?"
        val selectionArgs = arrayOf(id.toString())
        val cursor = db.query(
            ProductContract.ProductEntry.TABLE_NAME,
            projection,
            selection,
            selectionArgs,
            null,
            null,
            null
        )
//        val cursor = db.rawQuery("SELECT image_filename FROM ${ProductContract.ProductEntry.TABLE_NAME} WHERE ${BaseColumns._ID}=$id", null)
        var imageHash: String = ""

        with (cursor) {
            while(moveToNext()) {
                val productId = getInt(getColumnIndexOrThrow(BaseColumns._ID))
                val productImageHash = getString(getColumnIndexOrThrow(ProductContract.ProductEntry.IMAGE_FILENAME))
                imageHash = productImageHash
            }
        }
        Log.d("QWERTYUIOP", "Deleting image with hash: $imageHash and $id")
        val picturesDir = File(context.filesDir, "pictures")
        val thumbnailsDir = File(context.cacheDir, "thumbnails")

        File(picturesDir, "$imageHash.png").delete()
        File(thumbnailsDir, "$imageHash.webp").delete()
        Log.d("QWERTYUIOP", BaseColumns._ID + "=" + id)
        db.delete(ProductContract.ProductEntry.TABLE_NAME, BaseColumns._ID + "=" + id, null)
    }

    companion object {
        const val DATABASE_VERSION = 1
        const val DATABASE_NAME = "database.db"
    }
}